@@ -24,30 +24,66 @@
  * in together.
  */
 
+#include <features.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <paths.h>
 #include "config.h"
 
+#ifdef __UCLIBC_HAS_THREADS__
+#include <pthread.h>
+static pthread_mutex_t mylock = PTHREAD_MUTEX_INITIALIZER;
+# define LOCK   pthread_mutex_lock(&mylock)
+# define UNLOCK pthread_mutex_unlock(&mylock);
+#else       
+# define LOCK
+# define UNLOCK
+#endif      
+
+#ifdef __UCLIBC_HAS_THREADS__
+#include <pthread.h>
+extern pthread_mutex_t __getgrent_lock;
+# define GRENT_LOCK   pthread_mutex_lock(&__getgrent_lock)
+# define GRENT_UNLOCK pthread_mutex_unlock(&__getgrent_lock);
+#else
+# define GRENT_LOCK
+# define GRENT_UNLOCK
+#endif
+
 static int grp_fd = -1;
+static char *line_buff = NULL;
+static char **members = NULL;
 
 void setgrent(void)
 {
-	if (grp_fd != -1)
-		close(grp_fd);
-	grp_fd = open(_PATH_GROUP, O_RDONLY);
+    LOCK;
+    if (grp_fd != -1)
+	close(grp_fd);
+    grp_fd = open(_PATH_GROUP, O_RDONLY);
+    UNLOCK;
 }
 
 void endgrent(void)
 {
-	if (grp_fd != -1)
-		close(grp_fd);
-	grp_fd = -1;
+    LOCK;
+    if (grp_fd != -1)
+	close(grp_fd);
+    grp_fd = -1;
+    UNLOCK;
 }
 
 struct group *getgrent(void)
 {
-	if (grp_fd == -1)
-		return NULL;
-	return __getgrent(grp_fd);
+    struct group *r;
+
+    LOCK;
+    if (grp_fd == -1) {
+	UNLOCK;
+	return NULL;
+    }
+    UNLOCK;
+    GRENT_LOCK;
+    r = __getgrent(grp_fd, line_buff, members);
+    GRENT_UNLOCK;
+    return r;
 }
